#include <stdlib.h>
#include <string.h>
#include "basic_type_defs.h"
#include "game.h"
#include "automaton.h"
#include "proba_data.h"



/* The number of elapsed seconds must be reinitialised each time we start a new
   game or new level. */
struct game global_game ;
enum language language = Lang_French ;

/*
#define ANY_VOYELLE_LEN 6
const Letter any_voyelle[ANY_VOYELLE_LEN]
  __attribute__ ((section (".rodata"))) = {
  'a', 'e', 'i', 'o', 'u', 'y'
} ;

#define ANY_CONSONNE_LEN 20
const Letter any_consonne[ANY_CONSONNE_LEN]
  __attribute__ ((section (".rodata"))) = {
  'b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'q', 'r', 's',
  't', 'v', 'w', 'x', 'z'
} ;;
*/




static Letter proba_to_letter (unsigned long rnd)
{
  const unsigned long *reference_array ;
  short i ;
  Letter to_return = 'z' ;

  switch (language) {
  case Lang_French: reference_array = french_probas ; break ;
  case Lang_English: reference_array = english_probas ; break ;
  default:
    popup_message (FALSE, "proba_to_letter\nAssert 0.") ;
    return (to_return) ;
  }

  for (i = 0; i < 26; i++) {
    if (rnd <= reference_array[i]) {
      to_return = (Letter) ('a' + i) ;
      break ;
    }
  }
  
  return (to_return) ;
}





/* Détermine les lettres du jeu par effet de bord. */
static void choose_letters (struct game *game)
{
  short i ;
  unsigned long rnd ;

  for (i = 0; i < NB_LETTERS; i++) {
    /* Au hasard entre 1 et 1000000. */
    rnd = 1 + (rand ()) % 1000000 ;
    game->letters[i] = proba_to_letter (rnd) ;
    game->letters_backup[i] = game->letters[i] ;
  }
}





/* Fonction qui recopie un tableau sauf l'Èlement dÈsignÈ par l'index. */
static void copy_but (Letter *src, short src_len,
		      Letter *dest, short except_at)
{
  short i ;
  short dest_index = 0 ;

  for (i = 0; i < src_len; i++) {
    if (i != except_at) {
      dest[dest_index] = src[i] ;
      dest_index++ ;
    }
  }
}



static struct list_item* list_append (struct list_item *as_head,
				      struct list_item *as_tail)
{
  struct list_item *walker ;

  /* Recherche de la queue de la liste de tÍte. */
  if (as_head == NULL) return (as_tail) ;
  if (as_tail == NULL) return (as_head) ;

  walker = as_head ;
  while (walker->next != NULL) walker = walker->next ;
  walker->next = as_tail ;
  if (as_tail->prev != NULL) popup_message (FALSE, "list_append\nAssert 0") ;
  as_tail->prev = walker ;
  return (as_head) ;
}




/* Retourne la liste chaÓnÈe des possibilitÈs de mots. Pas d'effet de bord. */
static struct list_item* combine (Letter *letters, short len_letters,
				  unsigned long base, String prefix,
				  short len_prefix)
{
  Letter letters2[NB_LETTERS] ;
  short i, range ;
  String_letter tmp_char ;
  DAGElement bucket ;
  struct list_item *list_item = NULL ;
  struct list_item *returned_list_item = NULL ;

  if ((base != END_TRANSITION) && (len_letters > 0)) {
    /* Tableau avec 1 lettre de moins. */
    for (range = 0 ; range < len_letters; range++) {
      /* Code ASCII de la lettre choisie. */
      i = (short) letters[range] ;

      if (base + i < automaton_size) {
	bucket = automaton[base + i] ;
	tmp_char = GETCHAR (bucket) ;
	/* On vÈrifie qu'il y a bien transition, */
	/* et donc que le champ est utilisÈ.     */
	if (tmp_char == i) {
	  prefix[len_prefix] = tmp_char ;
	  /* On vÈrifie si le dagelement est final */
	  /* et si le mot fait au moins 3 lettres. */
	  if ((GETISFINAL (bucket)) && (len_prefix >= 2)) {
	    /* Ajouter le mot ‡ la liste des possibilitÈs. */
	    prefix[len_prefix + 1] = '\0' ;
	    list_item =
	      (struct list_item*) malloc (sizeof (struct list_item)) ;
	    if (list_item == NULL) {
	      popup_message (FALSE, "MÈmoire insuffisante.") ;
	      FREEZE_ON_FAILURE ;
	    }
	    list_item->prev = NULL ;
	    list_item->item =
	      (String) malloc ((len_prefix + 1) * sizeof (unsigned char)) ;
	    if (list_item->item == NULL) {
	      free (list_item) ;
	      popup_message (FALSE, "MÈmoire insuffisante.") ;
	      FREEZE_ON_FAILURE ;
	    }
	    strcpy (list_item->item, prefix) ;
	    /* ChaÓnage (ajout en tÍte). */
	    list_item->next = returned_list_item ;
	    if (returned_list_item != NULL) {
	      if (returned_list_item->prev != NULL)
		popup_message (FALSE, "combine\nAssert 0") ;
	      returned_list_item->prev = list_item ;
	    }
	    returned_list_item = list_item ;
	  }

	  copy_but (letters, len_letters, letters2, range) ;
	  list_item =
	    combine (letters2, (len_letters - 1), GETNEXTBASE (bucket), prefix,
		     (len_prefix + 1)) ;

	  /* ChaÓnage. */
	  returned_list_item = list_append (list_item, returned_list_item) ;
	}      /* Fin de if (tmp_char == i) */
      }        /* Fin de if (base + i < automaton_size) */
    }           /* Fin de for (range = 0 ; range < (len_letters - 1) ... */
  }         /* Fin de if ((base != END_TRANSITION) && ... */

  return (returned_list_item) ;
}





/* Retourne par effet de bord la longueur de la liste et le nombre */
/* total de caractËres de toutes les possibilitÈs cumulÈes.        */
static struct list_item* remove_double (struct list_item *list_items,
					short *list_length,
					short *total_list_length)
{
  struct list_item *returned_list_item ;  /* Nouvelle tÍte de liste. */
  struct list_item *walker ; /* Pour parcourir la liste vers la fin. */
  struct list_item *hunted ; /* Pour parcourir la liste vers le dÈbut. */
  short string_length ;

  walker = list_items ;
  returned_list_item = list_items ;
  *list_length = 0 ;
  *total_list_length = 0 ;

  while (walker != NULL) {
    /* On parcourt la liste vers la fin. */
    string_length = (short) strlen (walker->item) ;
    (*list_length)++ ;
    (*total_list_length) = (*total_list_length) + string_length ;
    hunted = walker->prev ;
    while (hunted != NULL) {
      /* On recherche des doubles vers le dÈbut de la liste. */
      if (strcmp (hunted->item, walker->item) == 0) {
	/* Doublon trouvÈ : on le supprime. */
	(*list_length)-- ;
	(*total_list_length) = (*total_list_length) - string_length ;
	if (hunted->prev == NULL) {
	  /* Le doublon Ètait la tÍte de liste */
	  returned_list_item = hunted->next ;
	  hunted->next->prev = NULL ;
	}
	else {
	  hunted->prev->next = hunted->next ;
	  hunted->next->prev = hunted->prev ;
	}
	free (hunted->item) ;
	free (hunted) ;
	break ;  /* Pas la peine de regarder plus en avant inductivement. */
      }          /* Fin de if (strcmp (hunter->item, walker->item) == 0) */
      hunted = hunted->prev ;
    }            /* Fin while (hunted != NULL) */
    walker = walker->next ;
  }              /* Fin de while (walker != NULL) */

  if (*list_length < 0) popup_message (FALSE, "remove_double\nAssert 0") ;
  return (returned_list_item) ;
}





void set_possibilities (struct game *game)
{
  String_letter prefix_buffer[NB_LETTERS + 1] ;
  struct list_item *tmp_list_item = NULL ;
  short list_length, total_list_length ;

  /* Si on tombe sur une combinaison avec aucune solution, on recommence. */
  while (tmp_list_item == NULL) {
    tmp_list_item =
      combine (game->letters, NB_LETTERS, initial_state, prefix_buffer, 0) ;
  }

  /* On retire les doublons et on calcule la longueur de la liste. */
  tmp_list_item = remove_double (tmp_list_item, &list_length,
				 &total_list_length) ;
  game->possibilities = tmp_list_item ;
  game->nb_possibilities = list_length ;
  game->total_possibilities_length = total_list_length ;
}





/* Fonction qui mÈlange les ÈlÈmÈnts du tableau reprÈsentant les lettres */
/* autorisÈes restantes dans le jeu. Travaille par effet de bord.        */
void shuffle_letters (struct game *game)
{
  Letter *letters = game->letters ;
  short len_letters = game->nb_letters_left ;
  short i, j, k ;
  Letter t ;

  for (k = 0; k < len_letters; k++) {
    i = (short) ((rand ()) % len_letters) ;
    j = (short) ((rand ()) % len_letters) ;
    t = letters[i] ;
    letters[i] = letters[j] ;
    letters[j] = t ;
  }
}





static BoolTy string_mem (String word, struct list_item* list_items)
{
  struct list_item *list_item ;

  list_item = list_items ;
  while (list_item != NULL) {
    if (strcmp (list_item->item, word) == 0) return (TRUE) ;
    list_item = list_item->next ;
  }

  /* Si on arrive ici c'est que la mot n'a pas ÈtÈ trouvÈ dans la liste. */
  return (FALSE) ;
}





static void free_list_items (struct list_item *list_items)
{
  struct list_item *list_item, *future_next ;

  list_item = list_items ;
  while (list_item != NULL) {
    future_next = list_item->next ;
    if (list_item->item != NULL) free (list_item->item) ;
    free (list_item) ;
    list_item = future_next ;
  }
}





/* NÈcessaire pour Èviter de libÈrer de la mÈmoire que l'on    */
/* n'aurait pas allouÈe lors du tout premier lancement du jeu. */
void init_empty_game_structure ()
{
  global_game.possibilities = (struct list_item*) NULL ;
  global_game.submissions_head = (struct list_item*) NULL ;
  global_game.submissions_tail = (struct list_item*) NULL ;
}




/* Initialise la structure de jeu en faisant le mÈnage au prÈalable */
/* si nÈcessaire. */
void start_game (unsigned long initial_score, short initial_level)
{
  /* MÈnage prÈalable. */
  if (global_game.possibilities != NULL) {
    free_list_items (global_game.possibilities) ;
    global_game.possibilities = (struct list_item*) NULL ;
  }

  if (global_game.submissions_head != NULL) {
    free_list_items (global_game.submissions_head) ;
    global_game.submissions_head = (struct list_item*) NULL ;
    global_game.submissions_tail = (struct list_item*) NULL ;
  }

  global_game.game_status = Game_attempt ;
  global_game.level = initial_level ;

  choose_letters (&global_game) ;
  global_game.nb_letters_left = NB_LETTERS ;
  set_possibilities (&global_game) ;

  global_game.min_length_to_win =
    min (MIN_LEN ,
	 ((global_game.total_possibilities_length * MIN_PERCENT) / 100)) ;
  global_game.available_time =
    max ((BASE_AVAILABLE_SECONDS - (initial_level * 10)),
	 MIN_AVAILABLE_SECONDS) ;
  /* Start a fresh countdown. */
  global_game.elapsed_seconds = 0 ;
  
  global_game.submission[0] = (String_letter) '\0' ;
  global_game.nb_found = 0 ;
  global_game.total_found_length = 0 ;
  global_game.progress_percent = 0 ;

  global_game.score = initial_score ;
  /* Timer will be started aside. */
}





static enum submission_result check_game_status (struct game *game)
{
  /* VÈrification de l'Ètat du jeu. */
  if (game->nb_found == game->nb_possibilities) {
    game->game_status = Game_level_complete ;
    return (Level_fully_complete) ;  /* Plus de mots ‡ trouver. */
  }

  /* On teste si on vient juste d'atteindre la performance minimale. */
  if ((game->total_found_length >= game->min_length_to_win) &&
      (game->game_status != Game_extra)) {
    game->game_status = Game_extra ;
    return (Level_successful) ;
  }
  /* Sinon, c'est que l'on est encore en train d'atteindre la perf min. */
  return (Correct) ;
}



void reset_current_proposition (struct game *game)
{
  int i ;

  /* On restaure les lettres de dÈpart. */
  for (i = 0; i < NB_LETTERS; i++) game->letters[i] = game->letters_backup[i] ;

  /* On met la suggestion à vide. */
  game->submission[0] = '\0' ;
  game->nb_letters_left = NB_LETTERS ;
}



/* Met ‡ jour les infos du jeu et retourne le changement de status */
/* courant (si le mot est correct, si on a trouvÈ toutes les suggestions */
/* ou si l'on vient juste d'atteindre les suggestions minimales pour */
/* avoir le droit de passer au niveau suivant. */
static enum submission_result update_game_with_right_submission
(struct game *game)
{
  struct list_item *list_item ;
  short i, len_submission ;

  /* On restaure les lettres de dÈpart. */
  for (i = 0; i < NB_LETTERS; i++)
    game->letters[i] = game->letters_backup[i] ;

  len_submission = (short) strlen (game->submission) ;
  /* On Ètend la liste des suggestions acceptÈes. */
  list_item = (struct list_item*) malloc (sizeof (struct list_item)) ;
  if (list_item != NULL) {
    list_item->item =
      (String) malloc ((1 + len_submission) * sizeof (unsigned char)) ;
    if (list_item->item == NULL) free (list_item) ;
    else {
      strcpy (list_item->item, game->submission) ;
      /* ChaÓnage. */
      list_item->next = NULL ;
      if (game->submissions_tail == NULL) {
        game->submissions_head = list_item ;
        game->submissions_tail = list_item ;
        list_item->prev = NULL ;
      }
      else {
        list_item->prev = game->submissions_tail ;
        game->submissions_tail->next = list_item ;
        game->submissions_tail = list_item ;
      }
    }

    /* Un de plus de trouvÈ ! */
    game->nb_found++ ;
    game->total_found_length += len_submission ;
    global_game.progress_percent =
      (game->total_found_length * 100) / game->min_length_to_win ;
    /* On met la suggestion ‡ vide. */
    game->submission[0] = '\0' ;
    game->nb_letters_left = NB_LETTERS ;

    /* Rajoute 8 secondes au temps restant sans poru autant dépasser le
       temps imparti initial. */
    game->elapsed_seconds = max (0, (game->elapsed_seconds - 8)) ;
    
    /* Mise ‡ jour du score. */
    if (game->game_status == Game_extra) {
      if (len_submission == NB_LETTERS)
        game->score += 200 ;    /* Bonus. */
      else game->score += (len_submission * 20) ;
    }
    else {
      if (len_submission == NB_LETTERS)
        game->score += 100 ;    /* Bonus. */
      else
        game->score += (len_submission * 10) ;
    }

    return (check_game_status (game)) ;
  }

  /* Allocation de mÈmoire ratÈe. */
  popup_message (FALSE, "MÈmoire insuffisante.") ;
  FREEZE_ON_FAILURE ;
}





/* *************************************************************
   [Description]: VÈrifie qu'un mot est une solution acceptable.
   ************************************************************* */
enum submission_result process_user_submission (struct game *game)
{
  /* On doit vÈrifier que les lettres utilisÈes appartiennent bien */
  /* aux lettres utilisÈes et qu'elles ne sont pas utilisÈes en    */
  /* double. Puis on doit vÈrifier que le mot existe bien dans le  */
  /* dictionnaire.                                                 */
  if (strlen (game->submission) <= 2) {
    return (Too_Short) ;
  }
  else {
    if (string_mem (game->submission, game->submissions_head)) {
      return (Already_Sumbitted) ;                /* Mot dÈj‡ proposÈ. */
    }
    else {
      if (string_mem (game->submission, game->possibilities)) {
        /* Mot correct et non dÈj‡ proposÈ. */
	return (update_game_with_right_submission (game)) ;
      }   /* Fin de if (string_mem (game->submission, game->possibilities)) */
      else {
	return (Non_Existing) ;    /* Mot non prÈsent dans le dictionnaire. */
      }
    }
  }              /* fin du else de if (strlen (game->submission) <= 2) */

  /* Code mort normalement. */
  popup_message (FALSE, "process_user_input\nAssert 1.") ;
  return (Non_Existing) ;
}





BoolTy try_add_letter_to_submission (struct game *game, short letter_index)
{
  short i, avail_p ;
  Letter tmp_letters[NB_LETTERS] ;
  Letter choice ;

  /* On vÈrifie que la demande se fait bien sur une lettre disponible. */
  avail_p = (short) (game->nb_letters_left - (short) letter_index - 1) ;
  if (avail_p < 0) return (FALSE) ;
  choice = game->letters[letter_index] ;
  copy_but
    (game->letters, game->nb_letters_left, tmp_letters, letter_index) ;
  game->nb_letters_left-- ;
  /* Recopie du tableau temporaire de lettres */
  /* restantes dans la structure du jeu. */
  for (i = 0; i < game->nb_letters_left; i++)
    game->letters[i] = tmp_letters[i] ;

  /* Ajout de la lettre ‡ la suggestion du joueur. */
  game->submission[NB_LETTERS - game->nb_letters_left - 1] = choice ;
  game->submission[NB_LETTERS - game->nb_letters_left] = '\0' ;
  return (TRUE) ;
}





BoolTy try_remove_letter_from_submission (struct game *game)
{
  Letter to_remove ;

  if (game->nb_letters_left == NB_LETTERS) return (FALSE) ;
  to_remove = game->submission[NB_LETTERS - game->nb_letters_left - 1] ;
  game->submission[NB_LETTERS - game->nb_letters_left - 1] = '\0' ;
  game->letters[game->nb_letters_left] = to_remove ;
  game->nb_letters_left++ ;
  return (TRUE) ;
}

