#import "GameView.h"
#import "HiddenManagementView.h"
#include "basic_type_defs.h"
#include "game.h"
#include "automaton.h"
#include "persistent.h"

@implementation GameView

@synthesize score_label ;
@synthesize performance_label ;
@synthesize time_progressview ;
@synthesize propositions_text ;
@synthesize current_proposition_label ;
@synthesize button_reset_current_proposition ;
@synthesize button_letter0 ;
@synthesize button_letter1 ;
@synthesize button_letter2 ;
@synthesize button_letter3 ;
@synthesize button_letter4 ;
@synthesize button_letter5 ;
@synthesize button_letter6 ;
@synthesize button_ok ;
@synthesize button_back ;
@synthesize button_shuffle ;
@synthesize button_surrender ;
@synthesize timer_game ;



- (void) start_game_timer
{
  timer_game =
  [NSTimer
   scheduledTimerWithTimeInterval:1.0 target: self
   selector: @selector (decrease_game_available_time) userInfo: nil
   repeats: YES] ;
}



- (void) stop_game_timer
{
  if (timer_game != nil) {
    [timer_game invalidate] ;
    timer_game = nil ;
  }
}



enum ActiveRequesterKind {
  ARK_none,
  ARK_Game_over,
  ARK_Surrender_no_minimal_perf,
  ARK_Surrender_with_minimal_perf
};



static int surrender_opened_requester = ARK_none ;

- (void) decrease_game_available_time
{
  /* Protect the section because it seems that invalidating the timer doens't
     stop it accurately: we still have one event sent after invalidation :/ */
  if (global_game.game_status != Game_over) {
    /* Increment the number of elapsed seconds. */
    global_game.elapsed_seconds++ ;
    /* Update the progress bar accordingly. */
    float elapsed_time_percent =
      ((float) global_game.elapsed_seconds) / ((float) global_game.available_time) ;
    [time_progressview setProgress: (1.0 - elapsed_time_percent)] ;
    /* Check if the game is over because of end of available time. */
    if (global_game.elapsed_seconds >= global_game.available_time) {
      [self stop_game_timer] ;
      global_game.game_status = Game_over ;
      UIAlertView *alert =
      [[UIAlertView alloc]
       initWithTitle:
        NSLocalizedString (@"str_you_know_what", @"You know what...")
       message:
        NSLocalizedString (@"str_elapsed_time", @"Elapsed time, so... game over!")
       delegate: self
       cancelButtonTitle: NSLocalizedString (@"str_i_know", @"I know!")
       otherButtonTitles: nil] ;
      [alert show] ;
      [alert release] ;
      /* Notify that a new requester has popped. */
      surrender_opened_requester = ARK_Game_over ;
    }
  }
}



+ (void) showSimpleMessage: (NSString*) text
{
  UIAlertView *alert =
    [[UIAlertView alloc]
      initWithTitle: @"Ttttt..."
      message: text delegate: self cancelButtonTitle: @"Ok"
      otherButtonTitles: nil] ;
  [alert show] ;
  [alert release] ;
}



- (void) ihmRefreshLettersStuff
{
  int i ;
  char empty_string = ' ' ;
  
  /* Affichage de la suggestion. */
  NSString *str =
    [[NSString alloc] initWithBytes: global_game.submission
                      length: strlen (global_game.submission)
                      encoding: NSISOLatin1StringEncoding] ;
  current_proposition_label.text = str ;
  [str release] ;
  
  /* Affichage des lettres autorisées restantes. */
  for (i = 0; i < NB_LETTERS; i++) {
    if (i >= global_game.nb_letters_left)
      str =
        [[NSString alloc]
          initWithBytes: &empty_string
          length: 1 encoding: NSISOLatin1StringEncoding] ;
    else
      str =
        [[NSString alloc]
          initWithBytes: &global_game.letters[i]
          length: 1 encoding: NSISOLatin1StringEncoding] ;
    [button_letters_as_array[i] setTitle: str forState: UIControlStateNormal] ;
    [str release] ;
  }
}



- (void) ihmRefreshCounters
{
  /* The number of proposed words versus the maxinal number of possible
     words. */
  NSString *str =
    [[NSString alloc] initWithFormat: @"%3d/%3d",
                      global_game.nb_found, global_game.nb_possibilities] ;
  performance_label.text = str ;
  [str release] ;
  /* The score. */
  str = [[NSString alloc] initWithFormat: @"%7ld", global_game.score] ;
  score_label.text = str ;
  [str release] ;
}



- (void) ihmFullPrintListWithsubmissions
{
  NSString *str ;
  
  struct list_item *cell = global_game.submissions_head ;
  while (cell != NULL) {
    str =
      [[NSString alloc] initWithBytes: cell->item
                        length: strlen (cell->item)
                        encoding: NSISOLatin1StringEncoding] ;
    NSString *new_submissions =
      [propositions_text.text stringByAppendingString: str] ;
    new_submissions = [new_submissions stringByAppendingString: @"\n"] ;
    propositions_text.text = new_submissions ;
    [str release] ;
    
    cell = cell->next ;
  }
}



- (void) ihmExtendListWithLastSubmission
{
  NSString *str =
    [[NSString alloc] initWithBytes: global_game.submissions_tail->item
                      length: strlen (global_game.submissions_tail->item)
                      encoding: NSISOLatin1StringEncoding] ;
  NSString *new_submissions =
    [propositions_text.text stringByAppendingString: str] ;
  new_submissions = [new_submissions stringByAppendingString: @"\n"] ;
  propositions_text.text = new_submissions ;
  [str release] ;
}



- (void) ihm_refresh_initial
{
  /* Clear the list of accepted submitted words. */
  propositions_text.text = @"" ;
  /* In case where the game was restaured, we must print the already submitted
     and accepted words. We just check this by looking at the submission_head
   pointer of the game. */
  [self ihmFullPrintListWithsubmissions] ;
  /* Re-init the progressbar. */
  [time_progressview setProgress: 1.0] ;
  [self ihmRefreshLettersStuff] ;
  [self ihmRefreshCounters] ;
}



- (IBAction) letter_button_pressed_callback: (id) sender
{
  int i ;
  
  for (i = 0; i < NB_LETTERS; i++) {
    if (sender == button_letters_as_array[i]) break ;
  }
  
  /* Arrived here, i is the index of the letter the pressed button corresponds
     to. */
  if (try_add_letter_to_submission (&global_game, i))
    [self ihmRefreshLettersStuff] ;
  else
    [GameView
      showSimpleMessage:
     NSLocalizedString
      (@"str_letter_cant_be_added", @"Letter can't be added to the proposition.")
    ] ;
}



- (IBAction) back_button_pressed_callback: (id) sender
{
  if (try_remove_letter_from_submission (&global_game))
    [self ihmRefreshLettersStuff] ;
  else
    [GameView
      showSimpleMessage:
        NSLocalizedString
          (@"str_no_more_letter_to_return", @"No more letter to return to the pool.")
     ] ;
}



- (IBAction) ok_button_pressed_callback: (id) sender
{
  switch (process_user_submission (&global_game)) {
  case Correct:
    [self ihmRefreshLettersStuff];
    [self ihmExtendListWithLastSubmission] ;
    [self ihmRefreshCounters] ;
    break ;
      
  case Too_Short:
    [GameView
      showSimpleMessage:
        NSLocalizedString
          (@"str_must_be_3_letters", @"Words must be 3 letters at least.")] ;
    break ;
      
  case Already_Sumbitted:
    [GameView
      showSimpleMessage:
        NSLocalizedString
          (@"str_word_already_proposed", @"Word already proposed.")] ;
    break ;
      
  case Non_Existing:
    [GameView showSimpleMessage:
      NSLocalizedString (@"str_unknown_word", @"Unknown word.")] ;
    break ;
      
  case Level_successful:
    /* We just reached the minimal performance to go to the next level. */
    [self ihmRefreshLettersStuff] ;
    [self ihmExtendListWithLastSubmission] ;
    [self ihmRefreshCounters] ;
    [GameView
      showSimpleMessage:
        NSLocalizedString
          (@"str_min_perf_reached_for_next_level",
           @"Minimal performance reached for the next level.")] ;
    break ;
      
  case Level_fully_complete:
    [GameView
      showSimpleMessage:
        NSLocalizedString
          (@"str_all_possible_found",
           @"All possible words found. Wrapping to the next level.")] ;
    [self stop_game_timer] ;
    start_game (global_game.score, (global_game.level + 1)) ;
    [self start_game_timer] ;
    [self ihm_refresh_initial] ;
    break ;
  }
}



- (IBAction) shuffle_button_pressed_callback: (id) sender
{
  shuffle_letters (&global_game) ;
  [self ihmRefreshLettersStuff] ;
}



- (IBAction) reset_current_proposition_button_pressed_callback: (id) sender
{
  reset_current_proposition (&global_game) ;
  [self ihmRefreshLettersStuff] ;
}



- (void) alertView: (UIAlertView*) actionSheet
clickedButtonAtIndex: (NSInteger) buttonIndex
{
  switch (surrender_opened_requester) {
  case ARK_Surrender_no_minimal_perf:
    /* Requester in case of surrendering without having reached the minimal
       performance. */
    if (buttonIndex != 0) {
      [self stop_game_timer] ;
      global_game.game_status = Game_over ;
      UIAlertView *alert =
        [[UIAlertView alloc]
         initWithTitle:
           NSLocalizedString (@"str_you_know_what", @"You know what...")
         message: NSLocalizedString (@"str_game_over", @"Game over!")
         delegate: self
         cancelButtonTitle: NSLocalizedString (@"str_i_know", @"I know!")
         otherButtonTitles: nil] ;
      [alert show] ;
      [alert release] ;
      /* Notify that a new requester has popped. */
      surrender_opened_requester = ARK_Game_over ;
    }
    break ;
      
  case ARK_Surrender_with_minimal_perf:
    /* Requester in case of surrendering with having reached the minimal
       performance. */
    [self stop_game_timer] ;
    surrender_opened_requester = ARK_none ;
    start_game (global_game.score, (global_game.level + 1)) ;
    [self start_game_timer] ;
    [self ihm_refresh_initial] ;
    break ;
      
  case ARK_Game_over:
    /* Update the hall of fame entries. If a high score is reach, pop a
       message before switching to the highscore view. */
    if (update_high_scores (&global_game)) {
      UIAlertView *alert =
        [[UIAlertView alloc]
          initWithTitle:
            NSLocalizedString (@"str_felicitation", @"Felicitation!")
          message:
            NSLocalizedString
              (@"str_you_got_highscore", @"You got a new highscore!")
          delegate: self
          cancelButtonTitle: NSLocalizedString (@"str_yep_i_good" ,@"Yep, I'm good!")
          otherButtonTitles: nil] ;
      [alert show] ;
      [alert release] ;
    }
    /* Switch to the highscore screen. */
    surrender_opened_requester = ARK_none ;
    [hidden_management_view show_highscore_view] ;
    break ;
      
  default:
    surrender_opened_requester = ARK_none ;
    break ;
  }
}



- (void) surrender_button_pressed_callback: (id) sender
{
  UIAlertView *alert ;
  
  switch (global_game.game_status) {
  case Game_attempt:
    surrender_opened_requester = ARK_Surrender_no_minimal_perf ;
    alert =
      [[UIAlertView alloc]
       initWithTitle:
          NSLocalizedString (@"str_min_perf_not_reached", @"Minimal performance not reached.")
        message:
          NSLocalizedString (@"str_surrender_anyway", @"Surrender anyway?")
        delegate: self
        cancelButtonTitle: NSLocalizedString (@"str_oups_no", @"Oups, no!")
        otherButtonTitles:
          NSLocalizedString (@"str_yep_i_m_tired", @"Yep, I' tired."), nil] ;
    [alert show] ;
    [alert release] ;
    break ;
      
  case Game_extra:
  case Game_level_complete:
    surrender_opened_requester = ARK_Surrender_with_minimal_perf ;
    alert =
      [[UIAlertView alloc]
        initWithTitle:
         NSLocalizedString
          (@"str_min_perf_reached", @"Minimal performance reached.")
        message:
          NSLocalizedString (@"str_go_to_next_level", @"Go to next level?")
        delegate: self
        cancelButtonTitle: NSLocalizedString (@"str_no_i_go_on", @"No, I go on.")
        otherButtonTitles:
          NSLocalizedString (@"str_yep", @"Yep!"), nil] ;
    [alert show] ;
    [alert release] ;
    break ;
      
  default:
    break ;
  }
}


- (IBAction) return_to_setup_view
{
  [hidden_management_view show_setup_view] ;
}


/* ***************************************************** */
/* **** BELOW ARE Cocoa OVERRIDEN METHODS FOR OUR STUFF. */
/* ***************************************************** */



/* Initial refresh. */
- (void) didMoveToSuperview
{
  button_letters_as_array[0] = button_letter0 ;
  button_letters_as_array[1] = button_letter1 ;
  button_letters_as_array[2] = button_letter2 ;
  button_letters_as_array[3] = button_letter3 ;
  button_letters_as_array[4] = button_letter4 ;
  button_letters_as_array[5] = button_letter5 ;
  button_letters_as_array[6] = button_letter6 ;
  [self ihm_refresh_initial] ;
  /* Since this event is triggered by the start of a new game, we start the
     timer. */
  [self start_game_timer] ;
}



- (void) dealloc
{
  // APPELEE ???
  [score_label dealloc] ;
  [performance_label dealloc] ;
  [time_progressview dealloc] ;
  [propositions_text dealloc] ;
  [current_proposition_label dealloc] ;
  [button_reset_current_proposition dealloc] ;
  [button_letter0 dealloc] ;
  [button_letter1 dealloc] ;
  [button_letter2 dealloc] ;
  [button_letter3 dealloc] ;
  [button_letter4 dealloc] ;
  [button_letter5 dealloc] ;
  [button_letter6 dealloc] ;
  [button_ok dealloc] ;
  [button_back dealloc] ;
  [button_shuffle dealloc] ;
  [button_surrender dealloc] ;
  [timer_game dealloc] ;
  
  [super dealloc] ;
}

@end
