//
//  IBlablaAppDelegate.m
//  IBlabla
//
//  Created by Didou on 07/11/09.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//
#import "GlobalITouchData.h"
#import "IBlablaAppDelegate.h"
#import "IBlablaViewController.h"
#include "basic_type_defs.h"
#include "game.h"
#include "automaton.h"
#include "persistent.h"

@implementation IBlablaAppDelegate

@synthesize window ;
@synthesize viewController ;



/* Consider incrementing the 2 last digits each time the preferences format
 is no mode compatible with the one of the previous version. */
#define PREFS_MAGIC_NUMBER TAGTOLONG('I','B','0','0')

/* *******************
 Kind: Not exported.
 ******************* */
+ (BOOL) initializePersistent
{
  NSUserDefaults *prefsDb = [NSUserDefaults standardUserDefaults] ;
  NSData *testIfPrefSaved = [prefsDb objectForKey: prefMagicNumberKey] ;
  BOOL saveStatus ;
	
  /* Verifies if some preferences were saved. */
  if (testIfPrefSaved != nil) {
    NSInteger magicNumber = [prefsDb integerForKey: prefMagicNumberKey] ;
    if (magicNumber == PREFS_MAGIC_NUMBER) return (YES) ;
  }
	
  /* Either there were not saved preferences or some but with a wrong magic
     number. Then creare some fresh and default ones. */
	NSNumber *magicNumber = [NSNumber numberWithInt: PREFS_MAGIC_NUMBER] ;
  NSData *hall_of_fame =
    [[NSData alloc]
      initWithBytes: &global_hall_of_fame 
      length: sizeof (struct HallOfFame)] ;
  enum language lang = Lang_French ;
  int nb_submissions = 0 ;
  int total_bytes_size = 0 ;
  NSData *game_structure = nil ;
  NSData *game_submissions = nil ;
  
  NSDictionary *iblablaAppPrefs =
    [[NSDictionary alloc]
     initWithObjectsAndKeys:
        magicNumber, prefMagicNumberKey,
        hall_of_fame, prefHallOfFameKey,
        lang, prefLanguageKey,
        nb_submissions, prefNbSubmissionsKey,
        total_bytes_size, prefTotalBytesSizeKey,
        game_structure, prefGameStructureKey,
        game_submissions, prefGameSubmissionsKey,
        nil] ;
  
  [prefsDb registerDefaults: iblablaAppPrefs] ;
  if ([prefsDb synchronize]) saveStatus = YES ;
	else {
    /* Synchronisation failed... */
		saveStatus = NO ;
	}
	
	[iblablaAppPrefs release] ;
  return (saveStatus) ;
}



+ (BOOL) savePersistent
{
  NSUserDefaults *prefsDb = [NSUserDefaults standardUserDefaults] ;
  
	/* Always save the magic number. */
	[prefsDb
   setInteger: (NSInteger) PREFS_MAGIC_NUMBER forKey: prefMagicNumberKey] ;

  /* The scores. */
  NSData *hall_of_fame =
    [[NSData alloc]
     initWithBytes: &global_hall_of_fame 
     length: sizeof (struct HallOfFame)] ;
  [prefsDb setObject: hall_of_fame forKey: prefHallOfFameKey] ;
  
  /* The realtime sage of the game.
     The game structure only if the game is currently running, otherwise
     save nil objects. */
  int nb_submissions = 0 ;
  int total_bytes_size = 0 ;
  NSData *game_structure = nil ;
  NSData *game_submissions = nil ;
 
  /* Initialise the game structure only if the game is still running. */
  if (global_game.game_status != Game_over) {
    game_structure =
      [[NSData alloc]
        initWithBytes: &global_game length: sizeof (struct game)] ;
    char *submissions_str =
      contiguous_strings_from_list
        (global_game.submissions_head, &nb_submissions, &total_bytes_size) ;
    game_submissions =
      [[NSData alloc]
       initWithBytes: submissions_str length: total_bytes_size] ;
    free (submissions_str) ;
  }
  
  /* Anyway, save the game structuren possibly with null and void data. */
  [prefsDb setInteger: (NSInteger) language forKey: prefLanguageKey] ;
  [prefsDb
    setInteger: (NSInteger) nb_submissions forKey: prefNbSubmissionsKey] ;
  [prefsDb
   setInteger: (NSInteger) total_bytes_size forKey: prefTotalBytesSizeKey] ;
  [prefsDb setObject: game_structure forKey: prefGameStructureKey] ;
  [prefsDb setObject: game_submissions forKey: prefGameSubmissionsKey] ;
  
  return ([prefsDb synchronize]) ;
}



+ (void) loadPersistent
{
  NSUserDefaults *prefsDb = [NSUserDefaults standardUserDefaults] ;
  
  /* Load the highscores. */
  NSData *hall_of_fame = [prefsDb dataForKey: prefHallOfFameKey] ;
  [hall_of_fame
   getBytes: &global_hall_of_fame length: sizeof (struct HallOfFame)] ;
  /* Save game will be examined in the setup view part. */
}



- (void) applicationDidFinishLaunching: (UIApplication*) application
{
  // Override point for customization after app launch
  srand ([[NSDate date] timeIntervalSince1970]) ;
  /* Handle the preferences. */
  [IBlablaAppDelegate initializePersistent] ;
  [IBlablaAppDelegate loadPersistent] ;
  /* Counter-act the strange shift of 20 vertical pixels between the UIBbuilder
     and the effective display. Shift the view 20 pixels toward the bottom. */
  CGRect frame = [viewController view].frame ;
  frame.origin.y = 20 ;
  [[viewController view] setFrame: frame] ;
  
  [window addSubview: viewController.view] ;
  [window makeKeyAndVisible] ;
}



- (void) applicationWillTerminate: (UIApplication *) application
{
  [IBlablaAppDelegate savePersistent] ;
}



- (void) dealloc
{
  [viewController release] ;
  [window release] ;
  [super dealloc] ;
}

@end
