#import "GlobalITouchData.h"
#import "HiddenManagementView.h"
#import "SetupView.h"
#include "basic_type_defs.h"
#include "game.h"
#include "automaton.h"
#include "persistent.h"


@implementation SetupView

@synthesize button_go_french ;
@synthesize button_go_english ;
@synthesize button_show_hall_of_fame ;


- (IBAction) go_button_pressed_callback : (id) sender
{
  if (sender == button_go_french) {
    language = Lang_French ;
    load_dictionary (Lang_French) ;
  }
  else {
    language = Lang_English ;
    load_dictionary (Lang_English) ;
  }
  init_empty_game_structure () ;
  start_game (/*initial_score*/ 0, /*initial_level*/0) ;
  /* Bringing the game view on the top will directly trigger the initial
     refresh and timer by side effect. */
  [hidden_management_view show_game_view] ;
}



- (IBAction) show_hall_of_fame_button_pressed_callback : (id) sender
{
  [hidden_management_view show_highscore_view] ;
}



- (void) check_saved_game_and_restart
{
  NSUserDefaults *prefsDb = [NSUserDefaults standardUserDefaults] ;
  
  /* If there is a saved game, then directly restore it. Note that since we
     only save running games or nil, if we get something different from nil
     that's the game was running. */
  NSData *game_structure =
    [[NSUserDefaults standardUserDefaults] objectForKey: prefGameStructureKey] ;
  if (game_structure != nil) {
    /* There is a saved game. Load it. Be careful that pointers are dangling
       and data is pretty mangled. We will correct it just later. */
    [game_structure getBytes: &global_game length: sizeof (struct game)] ;
    /* The language stuff. */
    NSInteger t = [prefsDb integerForKey: prefLanguageKey] ;
    language = (enum language) t ;
    /* Now submissions stuff. */
    t = [prefsDb integerForKey: prefNbSubmissionsKey] ;
    int nb_submissions = (int) t ;
    /* Only process the submissions if there are some. */
    if (nb_submissions > 0) {
      t = [prefsDb integerForKey: prefTotalBytesSizeKey] ;
      int total_bytes_size = (int) t ;
      NSData *game_submissions =
        [[NSUserDefaults standardUserDefaults]
          objectForKey: prefGameSubmissionsKey] ;
      /* Alloc enough data to recover the submissions. */
      char *submissions_str = (char*) malloc (total_bytes_size * sizeof (char)) ;
      /* Convert data from object to bytes. */
      [game_submissions
       getBytes: submissions_str length: (total_bytes_size * sizeof (char))] ;
      /* Now, convert the big submissions string into a linked list and set in
         place the tail of the list in the game structure. */
      global_game.submissions_head =
        list_from_contiguous_strings
          (submissions_str, nb_submissions, &global_game.submissions_tail) ;
      free (submissions_str) ;
    }    /* End of if (nb_submissions > 0). */
    else {
      global_game.submissions_head = NULL ;
      global_game.submissions_tail = NULL ;
    }
    
    /* Restaure the dictionary before re-computing possibilities. */
    if (language == Lang_French) load_dictionary (Lang_French) ;
    else load_dictionary (Lang_English) ;
    
    /* Make sure to return all the letters to the pool in order to be able to
       compute again the possibilities. */
    reset_current_proposition (&global_game) ;
    
    /* Now rebuild the list of possibilities. */
    set_possibilities (&global_game) ;
    
    /* Kill the saved data to prevent restarting the saved game several
       times. To do so, set the saved game structure entry to nil. */
    [prefsDb setObject: nil forKey: prefGameStructureKey] ;
    [prefsDb synchronize] ;
    
    /* Directly start the game. Timer and initial refresh will be trigered by
       bringing the game view on the top. */
    [hidden_management_view show_game_view] ;
  }      /* End of if (game_structure != nil). */
}


- (void) didMoveToSuperview
{
  [self check_saved_game_and_restart] ;
}
@end
